package com.bcxin.event.core;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bcxin.event.core.exceptions.BadEventException;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.Optional;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class FlinkConstants {
    private static final Logger logger = LoggerFactory.getLogger(FlinkConstants.class);
    private final static Collection<String> SYS_DATETIME_FIELD_NAMES = Stream.of("LASTMODIFIED", "CREATED").collect(Collectors.toList());
    private final static Collection<String> DATETIME_FIELD_NAMES = Stream.of("TIME", "DATE", "FZRQ","FIRST_ISSUE_SERVICE_PERMIT").collect(Collectors.toList());

    public static final String REDIS_HMAP_DB_NAME_FORMAT = "%s:%s:%s_%s";
    public static final String EMPLOYEE_STATION_ID = "security_station_id";
    public static final String EMPLOYEE_STATION_NAME = "security_station_name";
    public static final String EMPLOYEE_STATION_SUPERVISE_DEPART_ID = "security_supervise_depart_id";
    public static final String EMPLOYEE_STATION_SUPERVISE_DEPART_ID_INDEX = "security_supervise_depart_id_index";
    public static final String EMPLOYEE_STATION_SUPERVISE_DEPART_NAME = "security_supervise_depart_name";
    public static final String SUPERVISE_DEPART_ID = "supervise_depart_id";
    public static final String SUPERVISE_DEPART_NAME = "supervise_depart_name";
    public static final String SUPERVISE_REGIONId = "supervise_region_id";

    public static int getCalculatedParallelism() {
        int value = (int) Math.ceil(KafkaConstants.DEFAULT_PARTITION_COUNT * 1.0 / 5);

        if (value < 5) {
            value = 5;
        }

        if (value > 10) {
            value = 10;
        }

        return value;
    }

    public static String formatValue(String nodeKey, String nodeValue) {
        if (nodeValue == null) {
            return null;
        }

        if (nodeValue instanceof String) {
            String bsStr = nodeValue;
            if (isDateTimeField(nodeKey) && (bsStr.contains("0000-00-00") || bsStr.contains("9999-"))) {
                return null;
            }
        }

        try {
            if (isDateTimeField(nodeKey)) {
                if (!StringUtils.isEmpty(nodeValue) && !nodeValue.contains("-") && StringUtils.isNumeric(nodeValue)) {
                    return (new SimpleDateFormat("yyyy-MM-dd hh:mm:ss").format(new Date(getTimeValue(nodeKey, nodeValue))));
                } else {
                    if (StringUtils.isEmpty(nodeValue) ||
                            com.bcxin.event.core.utils.StringUtils.hasAlpha(nodeValue)) {
                        return null;
                    }

                    if (nodeValue.contains("999-") || nodeValue.contains("000-")) {
                        return null;
                    }

                    return nodeValue;
                }
            }
        } catch (Exception ex) {
            logger.error("可忽略: formatValue.key={};value={};转换出错=%s;尝试转为Double", nodeKey, nodeValue, ex);
            ex.printStackTrace();
        }

        return nodeValue;
    }

    public static long getTimeValue(String key, Object value) {
        try {
            return Long.parseLong(String.valueOf(value));
        } catch (Exception ex) {
            try {
                return (long) Double.parseDouble(String.valueOf(value));
            } catch (Exception e) {
                return 0;
            }
        }
    }

    public static boolean isDateTimeField(String nodeKey) {
        return FlinkConstants.SYS_DATETIME_FIELD_NAMES.stream().anyMatch(ix -> ix.equalsIgnoreCase(nodeKey)) ||
                FlinkConstants.DATETIME_FIELD_NAMES.stream().anyMatch(ix -> nodeKey.toUpperCase().contains(ix));
    }

    public static String getCalculatedParallelismKey(String value) {
        if (StringUtils.isEmpty(value)) {
            return value;
        }

        /**
         * 最大设置为3个分区（总共有55个进行订阅）=所以总共需要55*3= 165（我flink-conf.yaml设置了总共的slot为168）
         */
        final int maxParallelismCount = KafkaConstants.DEFAULT_PARTITION_COUNT;

        int parallelValue = KafkaConstants.getCalculableHashValue(Math.abs(value.hashCode())) % maxParallelismCount;

        return String.valueOf(parallelValue);
    }

    public static String getExtractedParallelismOriginalKey(JSONObject dataNode) {
        Object parallelismKeyValue = null;
        /**
         * 默认通过组织来分区
         */
        StringBuilder sb = new StringBuilder();
        Optional<String> keyOptional = dataNode.keySet()
                .stream().filter(ix -> ix.equalsIgnoreCase("domainId") ||
                        ix.replace("_", "")
                                .equalsIgnoreCase("organizationId"))
                .findFirst();

        /**
         * 比如职员/保安公司等
         */
        sb.append(String.format("第一个keyOption=%s", keyOptional));
        if (keyOptional.isPresent()) {
            parallelismKeyValue = dataNode.get(keyOptional.get());
            sb.append(String.format("第一个keyOption的值=%s", parallelismKeyValue));
        } else {
            /**
             * 针对比如: tenant_user_credenti....这类的数据
             */
            keyOptional = dataNode.keySet()
                    .stream().filter(ix -> ix.equalsIgnoreCase("tenant_user_id"))
                    .findFirst();
            sb.append(String.format("第二个keyOption=%s", keyOptional));
            if (keyOptional.isPresent()) {
                parallelismKeyValue = dataNode.get(keyOptional.get());
                sb.append(String.format("第二个keyOption的值=%s", parallelismKeyValue));
            }
            else {
                keyOptional = dataNode.keySet()
                        .stream().filter(ix -> ix.equalsIgnoreCase("id") ||
                                ix.replace("_", "").equalsIgnoreCase("pkId"))
                        .findFirst();
                sb.append(String.format("第三个keyOption=%s", keyOptional));
                if (keyOptional.isPresent()) {
                    parallelismKeyValue = dataNode.get(keyOptional.get());
                    sb.append(String.format("第三个keyOption的值=%s", parallelismKeyValue));
                }
            }
        }

        if (parallelismKeyValue == null) {
            keyOptional =
                    dataNode.keySet().stream().filter(
                            ix -> ix.equalsIgnoreCase("id") ||
                                    ix.equalsIgnoreCase("tenant_user_id")).findFirst();
            sb.append(String.format("第四个keyOption=%s", keyOptional));
            if (keyOptional.isPresent()) {
                parallelismKeyValue = dataNode.get(keyOptional.get());
                sb.append(String.format("第三个keyOption的值=%s", parallelismKeyValue));
            }


            if (parallelismKeyValue == null) {
                Object id = dataNode.get("id");
                if (id == null) {
                    logger.info(sb + ";取默认default(=NULL)无效值" + "无效V5的分区选项: " + JSON.toJSONString(dataNode));

                    return "default";
                }

                return String.valueOf(id);
            }
        }



        return String.valueOf(parallelismKeyValue);
    }
}
